
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% CONTROLLO DEI PROCESSI %%%
%%%%%% PROBLEMA TIPO 'A' %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all 
clc

global g a1 a2 a3 a4 A1 A2 A3 A4 gamma1 gamma2 k1 k2
global gamma1_ gamma2_ k1_ k2_


%% Parametri dei controllori

% H-
kp1_=4.4;
ki1_=0.09;
kp2_=5.5;
ki2_=0.08;

% H+
kp1=4.5;
ki1=0.09;
kp2=-0.1;
ki2=-0.001;

%% Stati iniziali per simulazioni

% livello liquido nei serbatoi [cm]
hiniz1=[7;7;7;7];

%% Parametri del modello

g=981; % [cm/s^2]
kc=0.5;
% sezioni serbatoi [cm^2]
A1=28;
A2=32;
A3=28;
A4=32;
% sezioni fori [cm^2]
a1=0.071;
a2=0.057;
a3=0.071;
a4=0.057;
% parametri del punto di equilibrio H-
k1_=3.33;
k2_=3.35;
gamma1_=0.7;
gamma2_=0.6;
% parametri del punto di equilibrio H+
k1=3.14;
k2=3.29;
gamma1=0.43;
gamma2=0.34;
% tensioni di equilibrio [V]
v1_=3;
v2_=3;
v1=3.15;
v2=3.15;


%% 1) Dinamica del sistema

% Le equazioni dinamiche del sistema sono implementate
% nelle funzioni matlab qTank_.m e qTank.m con i parametri
% del punto H- e H+ rispettivamente.


%% 2) Stati di equilibrio

h3_e=(((1-gamma2_)*k2_*v2_)/(sqrt(2*g)*a3))^2;
h4_e=(((1-gamma1_)*k1_*v1_)/(sqrt(2*g)*a4))^2;
h1_e=((sqrt(2*g*h3_e)*a3+gamma1_*k1_*v1_)/(sqrt(2*g)*a1))^2;
h2_e=((sqrt(2*g*h4_e)*a4+gamma2_*k2_*v2_)/(sqrt(2*g)*a2))^2;
HstatoRegime_=[h1_e;h2_e;h3_e;h4_e]

h3e=(((1-gamma2)*k2*v2)/(sqrt(2*g)*a3))^2;
h4e=(((1-gamma1)*k1*v1)/(sqrt(2*g)*a4))^2;
h1e=((sqrt(2*g*h3e)*a3+gamma1*k1*v1)/(sqrt(2*g)*a1))^2;
h2e=((sqrt(2*g*h4e)*a4+gamma2*k2*v2)/(sqrt(2*g)*a2))^2;
HstatoRegime=[h1e;h2e;h3e;h4e]


%% 3) Linearizzazione del sistema

% matrice H
H_=[kc 0 0 0; 0 kc 0 0];
H=H_;

%% P-

% matrice F
f11_= -a1/A1*sqrt(g/(2*h1_e));
f22_= -a2/A2*sqrt(g/(2*h2_e));
f33_= -a3/A3*sqrt(g/(2*h3_e));
f44_= -a4/A4*sqrt(g/(2*h4_e));
f13_= a3/A1*sqrt(g/(2*h3_e));
f24_= a4/A2*sqrt(g/(2*h4_e));
F_=[f11_ 0 f13_ 0; 0 f22_ 0 f24_; 0 0 f33_ 0; 0 0 0 f44_];

% matrice G
g11_= gamma1_*k1_/A1;
g22_= gamma2_*k2_/A2;
g32_= (1-gamma2_)*k2_/A3;
g41_= (1-gamma1_)*k1_/A4;
G_=[g11_ 0; 0 g22_; 0 g32_; g41_ 0];

% matrice di trasferimento
s=tf('s');
W_=(H_*((s*eye(4))-F_)^-1)*G_;
W_=minreal(W_)

% costanti di tempo
T_=-1./pole(W_)
Zeri_=zero(W_)


%% P+

% matrice F
f11= -a1/A1*sqrt(g/(2*h1e));
f22= -a2/A2*sqrt(g/(2*h2e));
f33= -a3/A3*sqrt(g/(2*h3e));
f44= -a4/A4*sqrt(g/(2*h4e));
f13= a3/A1*sqrt(g/(2*h3e));
f24= a4/A2*sqrt(g/(2*h4e));
F=[f11 0 f13 0; 0 f22 0 f24; 0 0 f33 0; 0 0 0 f44];

% matrice G
g11= gamma1*k1/A1;
g22= gamma2*k2/A2;
g32= (1-gamma2)*k2/A3;
g41= (1-gamma1)*k1/A4;
G=[g11 0; 0 g22; 0 g32; g41 0];

% matrice di trasferimento
W=H*(s*eye(4)-F)^-1*G;
W=minreal(W)
% costanti di tempo
T=-1./pole(W)
Zeri=zero(W)


%% 4) Simulazioni

%% mio

ss=Zeri(2)
guad_zero=H*(ss*eye(4)-F)^-1*G
[U,S,V]=svd(guad_zero)
dir=[V(2,1);V(2,2)]
guad_zero*dir

%% controllo decentralizzato

invW=W^-1
barG= [tf([1 -0.0127],[1 0.1]) 0;0 tf([1 -0.0127],[1 0.1])]
Cont_INT=[tf(-0.04,[1 0]) 0;0 tf(-0.04,[1 0])]
Cont_dec_simp=minreal(invW*barG*Cont_INT,0.0001)



sim mio_sim.mdl
